#!/usr/bin/perl -w

use strict;

package BSF;

use vars qw( $log $pkg $AUTOLOAD );

$pkg = "_bsfpackage";

open(JAVA, ">&", \*STDOUT) or die "Can't dup STDOUT: $!";
close STDOUT;
open(STDOUT, ">&", \*STDERR) or die "Can't replace STDOUT with dup STDERR: $!";

select(JAVA); $|++; select(STDOUT); $|++;

sub UNIVERSAL::import {
    my ($class) = @_;

    my $short = ($class =~ /([^:]+)$/)[0] || $class;

    no strict 'refs';
    my %h = %{$short . "::"};
    
    unless (exists $h{"AUTOLOAD"}) {
	*{$short . "::AUTOLOAD"} = sub {
	    my ($givenClass) = shift;
	    my $method = ($AUTOLOAD =~ /([^:]+)$/)[0] || $AUTOLOAD;

	    my $javaClassName = $class;
	    $javaClassName =~ s/::/./g;

	    my $beanName = $javaClassName . ".class";
	    my $javaClass = BSF::JavaObject->new($beanName);
	    return $javaClass->_callMethod($method, @_);
	};
    }
}

package BSF::Dumper;

use vars qw( @ISA );

require Data::Dumper;
@ISA = qw( Data::Dumper );

sub _dump {
    my($s, $val, $name) = @_;

    if (defined($val) && !ref($val)) {
	if ($val ne '' && (($val ^ $val) eq '0')) {
	    # number
	    return $val;
	} else {
	    # string
	    $val =~ s/([\\\'])/\\$1/g;
	    return '\'' . $val .  '\'';
	}
    }

    return $s->SUPER::_dump($val, $name);
}

package BSF::JavaObject;

use vars qw( $AUTOLOAD );
use overload '""' => "toString";

sub new {
    my ($this, $name) = @_;

    return bless(\$name, $this);
}

sub toString {
    return _callMethod(shift, "toString");
}

sub DESTROY {
    # do nothing...
}

sub AUTOLOAD {
    my $self = shift;

    my $method = $AUTOLOAD;
    $method =~ s/.*:://;

    return _callMethod($self, $method, @_);
}

sub _callMethod {
    my $self = shift;
    my $method = shift;

    my $hash = {};

    print BSF::JAVA "BEGIN-REQUEST\n";

    print BSF::JAVA $$self, "->", $method, " ";
    BSF::Gateway::dump_value(\@_);

    print BSF::JAVA "END-REQUEST\n";

    my $l = <>;
    if ($l =~ /BEGIN-RESPONSE/) {
	my $req = "";
	while (<>) {
	    last if /END-RESPONSE/;
	    
	    $req .= $_;
	}
	
	my $code = "package $BSF::pkg; no strict; { $req }";
	my $ret = eval($code);
	die "eval failed: $@" if $@;
	return $ret;
    }
}

package BSF::Gateway;

my $nextId = 0;

my $bsf = BSF::JavaObject->new("bsf");
{
    no strict 'refs';
    ${$BSF::pkg . "::bsf"} = $bsf;
}

my $l;
while(defined($l = <>)) {
    if ($l =~ /BEGIN-REQUEST/) {
	my $req = "";
	while (<>) {
	    last if /END-REQUEST/;
	    
	    $req .= $_;
	}

	my $code = "package $BSF::pkg; no strict; { $req }";
	my $response = eval($code);
	die "eval failed: $@" if $@;

	print BSF::JAVA "BEGIN-RESPONSE\n";

	dump_value($response);
	
	print BSF::JAVA "END-RESPONSE\n";
    }
}

sub dump_value
{
    my ($v) = @_;

#    my $hash = calculate_seen_hash(); # Why is this commented out?
    my $hash = {"bsf" => $bsf};

    register_subobjects($v, $hash);

    print BSF::JAVA format_value($v, $hash);
}

sub format_value
{
    my ($v, $hash) = @_;

    my $dumper = new BSF::Dumper ([$v]);
    
    $dumper->Terse(1);
    $dumper->Indent(1);
    $dumper->Useperl(1);

    $dumper->Seen($hash);

    return $dumper->Dump;
}

sub calculate_seen_hash
{
    no strict 'refs';

    my %hash = ();

    while (my($sym, $glob) = each %{$BSF::pkg . "::"}) {
	if (*$glob{SCALAR}) {
	    my $scalar = ${ *$glob{SCALAR} };
	    
	    if (defined($scalar) && ref($scalar) && (overload::StrVal($scalar) =~ /=/)) {
		$hash{$sym} = $scalar;
	    }
	}
    }

    return \%hash;
}

my %recursion_check = ();
sub register_subobjects
{
    my ($v, $hash) = @_;
    my $type = ref($v);
    
    return 1 unless defined $v;

    my $vLoc = overload::StrVal($v);

    return 0 if $recursion_check{$vLoc};
    local $recursion_check{$vLoc} = 1;

    if (!$type || $type eq "SCALAR") {
	# do nothing...
    } elsif ($type =~ /BSF::JavaObject/) {
	register_obj($v, $hash, "_JAVA" . $$v);
    } elsif ($type eq "REF") {
	register_subobjects($$v, $hash) || register_obj($v, $hash);
    } elsif ($type eq "ARRAY") {
	my $done = 1;
	foreach my $i (@$v) {
	    $done &&= register_subobjects($i, $hash);
	}
	register_obj($v, $hash) if (!$done)

    } elsif ($type eq "HASH") {
	my $done = 1;
	foreach my $i (values %$v) {
	    $done &&= register_subobjects($i, $hash);
	}
	register_obj($v, $hash) if (!$done)
    } else {
	register_obj($v, $hash);
    }

    return 1;
}

sub register_obj
{
    my ($v, $hash, $name) = @_;

    foreach my $i (values %$hash) {
	if (defined($i) && defined($v)) {
	    return if (overload::StrVal($i) eq overload::StrVal($v));
	}
    }

    if (!$name) {
	my $id = $nextId++;
	$name = "_obj$id";
    }

    no strict 'refs';
    ${$BSF::pkg . "::" . $name} = $v;
    $hash->{$name} = $v;
}

